"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getI18nMessages = exports.SpreadElement = void 0;
const tslib_1 = require("tslib");
const deepmerge_1 = tslib_1.__importDefault(require("deepmerge"));
const pluralize_1 = tslib_1.__importDefault(require("pluralize"));
const fat_snake_to_normal_1 = require("../../../util/fat-snake-to-normal");
const utils_1 = require("../../../../common/utils");
const change_case_1 = require("change-case");
// Need to preserve placeholder when parsing i18n file to js object and back
class SpreadElement {
    __type__ = "SpreadElement";
}
exports.SpreadElement = SpreadElement;
/**
 * Generate messages that should be added to app message pack (en.ts)
 * for translate entity resource name and fields.
 * Generated messages based on information from graphQL query - entity name and attributes.
 *
 * @param entityName - entity name string.
 * If entity name ends with 'Dto' or 'DTO' - this suffix will be removed in 'name' message.
 * @param attributes - graphQL attributes from graphQL query as GqlAttributeModel[]
 *
 * @returns messages as typescript object (react-admin i18n format),
 * which contains resource name, fields and enums:
 * {
 *  resource: {
 *    Entity: {
 *      name: "Entity |||| Entities",
 *      fields: {
 *        key: "value",
 *      },
 *    }
 *  },
 *  enums: {
 *    EnumTypeName: {
 *      VALUE_ONE: "Value one",
 *      VALUE_TWO: "Value two",
 *    }
 *  }
 * }
 *
 * Note: react-admin used `||||` as separator between singular and plural name forms.
 *
 */
function getI18nMessages(entityName, attributes) {
    const enumsMessages = getAllEnumsMessages(attributes);
    const resourcesMessages = getResourcesMessages(entityName, attributes);
    return (0, deepmerge_1.default)(resourcesMessages, enumsMessages);
}
exports.getI18nMessages = getI18nMessages;
// TODO test
function getAllEnumsMessages(attributes) {
    const enumsMessages = attributes
        .filter(attr => attr.enumOptions != null)
        .reduce((acc, attr) => ({ ...acc, [attr.gqlType.toString()]: getEnumMessages(attr) }), {});
    return { ...(Object.keys(enumsMessages).length > 0) ? { enums: enumsMessages } : undefined };
}
// TODO test; replace deprecated 'fatSnakeToNormal' with change-case function;
//  add test case for 'getAllEnumsMessages' where 'getEnumMessages' return 'undefined'
function getEnumMessages(attr) {
    return attr.enumOptions?.reduce((acc, enumValue) => ({ ...acc, [enumValue.name]: (0, fat_snake_to_normal_1.fatSnakeToNormal)(enumValue.value) }), {});
}
// TODO test, add test case where 'getResourcesMessages' return 'undefined'
function getResourcesMessages(entityName, attributes) {
    const entityNameWithoutDTO = (0, utils_1.removeDtoSuffixFromEntityName)(entityName);
    const resourcesMessages = {
        [entityName]: {
            name: `${(0, change_case_1.capitalCase)(entityNameWithoutDTO)} |||| ${(0, change_case_1.capitalCase)((0, pluralize_1.default)(entityNameWithoutDTO))}`,
            fields: attributes
                .reduce((acc, attr) => {
                // TODO we don't support case, where typeof attr.name is array of strings, throw exception in this case, add test
                const key = typeof attr.name === "string" ? attr.name : attr.name.join(".");
                // in the generated app single relation fields will refer to id for label
                if (attr.isSingleRelationField) {
                    return { ...acc, [key]: {
                            id: attr.displayName
                        } };
                }
                return { ...acc, [key]: attr.displayName };
            }, {})
        }
    };
    return {
        ...(Object.keys(resourcesMessages).length > 0) ? { resources: resourcesMessages } : undefined
    };
}
//# sourceMappingURL=getI18nMessages.js.map