"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getEntityAttributes = void 0;
const tslib_1 = require("tslib");
const graphql_1 = require("graphql");
const getTopFieldName_1 = require("./getTopFieldName");
const utils_1 = require("../../../../../common/utils");
const getAttributeNames_1 = require("./getAttributeNames");
const getEntityName_1 = require("./getEntityName");
const getTypeFields_1 = require("./getTypeFields");
const isAnyLeafType_1 = require("./isAnyLeafType");
const getSelectedFieldNamesFromTopField_1 = require("./getSelectedFieldNamesFromTopField");
const unwrapGraphQLType_1 = require("./unwrapGraphQLType");
const isFieldOfGraphQLNonNullType_1 = require("./isFieldOfGraphQLNonNullType");
const graphql_tag_1 = tslib_1.__importDefault(require("graphql-tag"));
const change_case_1 = require("change-case");
const DATE_TYPES = ['Date', 'Timestamp', 'LocalDateTime', 'LocalTime', 'DateTime', 'Time'];
const NUMBER_TYPES = ['Int', 'BigDecimal', 'BigInteger', 'Float', 'Long'];
function getEntityAttributes(documentNode, schema, idField, fileAttributesPropertiesAnswers = []) {
    // e.g. "petList"
    const queryFieldName = (0, getTopFieldName_1.getTopFieldName)(documentNode);
    // e.g. "PetDTO"
    const entityName = (0, getEntityName_1.getEntityName)(queryFieldName, schema);
    const typeFields = (0, getTypeFields_1.getTypeFields)(entityName, schema);
    const selectedFieldNames = (0, getSelectedFieldNamesFromTopField_1.getSelectedFieldNamesFromTopField)(documentNode);
    if (!selectedFieldNames.find(fieldKey => typeFields[fieldKey].name === idField)) {
        throw new Error(`Attributes of query '${queryFieldName}' did not contain id attribute with name '${idField}', ` +
            `which required for screen generation. If you are using entity with id field with name distinct from 'id', ` +
            `please be sure that you pass information about id field name in answers ('idField', 'listIdField', 'detailsIdField')`);
    }
    return selectedFieldNames
        // Do not include id attribute
        .filter(fieldKey => typeFields[fieldKey].name !== idField)
        // filter fileName properties
        .filter(fieldKey => !fileAttributesPropertiesAnswers.some(fileAttributesProperty => fileAttributesProperty?.fileNameSource === typeFields[fieldKey].name))
        .map(fieldKey => {
        const field = typeFields[fieldKey];
        const gqlType = (0, isFieldOfGraphQLNonNullType_1.isFieldOfGraphQLNonNullType)(field)
            ? (0, unwrapGraphQLType_1.unwrapGraphQLType)(field.type)
            : field.type;
        if (gqlType == null) {
            throw new Error(`GraphQL type is null for field '${field}' in query '${queryFieldName}'`);
        }
        const namedType = (0, graphql_1.getNamedType)(gqlType);
        const type = namedType.toString();
        const isEnumType = gqlType instanceof graphql_1.GraphQLEnumType;
        let fileProperties = fileAttributesPropertiesAnswers
            .map(fileAttributesProperty => ({
            ...fileAttributesProperty,
            downloadUrlQueryName: fileAttributesProperty.downloadUrlQuery
                ? (0, change_case_1.constantCase)((0, getTopFieldName_1.getTopFieldName)((0, graphql_tag_1.default)(fileAttributesProperty.downloadUrlQuery)))
                : undefined,
            uploadUrlQueryName: 'uploadUrlQuery' in fileAttributesProperty && fileAttributesProperty.uploadUrlQuery
                ? (0, change_case_1.constantCase)((0, getTopFieldName_1.getTopFieldName)((0, graphql_tag_1.default)(fileAttributesProperty.uploadUrlQuery)))
                : undefined,
        }))
            .find(fileAttributesProperty => fileAttributesProperty.fileIdSource === field.name);
        const isFileType = field.astNode?.directives?.[0]?.name.value === "fileId";
        if (isFileType && fileProperties == null) {
            fileProperties = { fileIdSource: field.name };
        }
        const required = (0, isFieldOfGraphQLNonNullType_1.isFieldOfGraphQLNonNullType)(field);
        return {
            kind: 'gql',
            name: field.name,
            displayName: (0, utils_1.capitalizeFirst)((0, utils_1.splitByCapitalLetter)(field.name)),
            isSingleRelationField: !(0, isAnyLeafType_1.isAnyLeafType)(gqlType) && !(gqlType instanceof graphql_1.GraphQLList),
            isMultiRelationField: !(0, isAnyLeafType_1.isAnyLeafType)(gqlType) && gqlType instanceof graphql_1.GraphQLList,
            type,
            gqlType,
            isDateType: DATE_TYPES.includes(type),
            isNumberType: NUMBER_TYPES.includes(type),
            isEnumType,
            isFileType,
            fileProperties,
            enumOptions: isEnumType ? gqlType.getValues() : undefined,
            nestedAttributes: (0, isAnyLeafType_1.isAnyLeafType)(gqlType) ? undefined : (0, getAttributeNames_1.getAttributeNames)(type, schema),
            required
        };
    });
}
exports.getEntityAttributes = getEntityAttributes;
//# sourceMappingURL=getEntityAttributes.js.map