"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const graphql_tag_1 = tslib_1.__importDefault(require("graphql-tag"));
const chai_1 = require("chai");
const getEntityAttributes_1 = require("./getEntityAttributes");
const test_commons_1 = require("../../../../../test/test-commons");
describe("getEntityAttributes", async () => {
    let schema;
    before(async () => {
        schema = await (0, test_commons_1.getMockSchema)();
    });
    it("derives a list of entity attributes", () => {
        const attributes = (0, getEntityAttributes_1.getEntityAttributes)(QUERY_NODE_PET_LIST, schema, "id");
        (0, chai_1.expect)(attributes
            .map(({ gqlType: _gqlType, ...attr }) => ({ ...attr })).sort())
            .to.deep.equal(EXPECTED_ATTRS_PET_LIST.sort());
    });
    it("relation field is marked as such", () => {
        const attributes = (0, getEntityAttributes_1.getEntityAttributes)(QUERY_NODE_PET_LIST, schema, "id");
        const ownerAttr = attributes.find(attr => attr.name === "owner");
        (0, chai_1.expect)(ownerAttr?.isSingleRelationField).to.eq(true);
    });
    it("relation field has entity name", () => {
        const attributes = (0, getEntityAttributes_1.getEntityAttributes)(QUERY_NODE_PET_LIST, schema, "id");
        const ownerAttr = attributes.find(attr => attr.name === "owner");
        (0, chai_1.expect)(ownerAttr?.type).to.eq("OwnerDTO");
    });
    it("should define correct relation type for mandatory relation field", function () {
        const attributes = (0, getEntityAttributes_1.getEntityAttributes)(QUERY_NODE_VISIT_LIST, schema, "id");
        const petAttr = attributes.find(attr => attr.name === "pet");
        (0, chai_1.expect)(petAttr.isSingleRelationField).to.be.true;
        (0, chai_1.expect)(attributes
            .map(({ gqlType: _gqlType, ...attr }) => ({ ...attr })).sort())
            .to.deep.equal([
            {
                kind: "gql",
                name: "description",
                displayName: "Description",
                isSingleRelationField: false,
                isMultiRelationField: false,
                required: false,
                type: "String",
                fileProperties: undefined,
                isFileType: false,
                isDateType: false,
                isEnumType: false,
                isNumberType: false,
                enumOptions: undefined,
                nestedAttributes: undefined,
            },
            {
                kind: "gql",
                name: "pet",
                displayName: "Pet",
                isSingleRelationField: true,
                isMultiRelationField: false,
                required: true,
                type: "PetDTO",
                fileProperties: undefined,
                isFileType: false,
                isDateType: false,
                isEnumType: false,
                isNumberType: false,
                enumOptions: undefined,
                nestedAttributes: [
                    "birthDate",
                    "description",
                    "diseases",
                    "id",
                    "identificationNumber",
                    "owner",
                    "tags",
                    "type",
                    "weightInGrams",
                    "passport",
                    "passportFilename",
                    "passportContentType",
                    "photo"
                ],
            },
            {
                kind: "gql",
                name: "visitEnd",
                displayName: "Visit End",
                isSingleRelationField: false,
                isMultiRelationField: false,
                required: false,
                type: "LocalDateTime",
                fileProperties: undefined,
                isFileType: false,
                isDateType: true,
                isEnumType: false,
                isNumberType: false,
                enumOptions: undefined,
                nestedAttributes: undefined
            },
            {
                kind: "gql",
                name: "visitStart",
                displayName: "Visit Start",
                isSingleRelationField: false,
                isMultiRelationField: false,
                required: false,
                type: "LocalDateTime",
                fileProperties: undefined,
                isFileType: false,
                isDateType: true,
                isEnumType: false,
                isNumberType: false,
                enumOptions: undefined,
                nestedAttributes: undefined
            }
        ]);
    });
    it("query contains id attribute with custom name", () => {
        const attributes = (0, getEntityAttributes_1.getEntityAttributes)(QUERY_WITHOUT_ID, schema, "identificationNumber");
        const dateAttr = attributes.find(attr => attr.name === "birthDate");
        (0, chai_1.expect)(dateAttr?.type).to.eq("Date");
    });
    it("fails if there is no 'id' attribute in query", () => {
        (0, chai_1.expect)(() => (0, getEntityAttributes_1.getEntityAttributes)(QUERY_WITHOUT_ID, schema, "id"))
            .to.throw((0, test_commons_1.stripNewLines)(`
Attributes of query 'pet' did not contain id attribute with name 'id', which required for screen generation. 
If you are using entity with id field with name distinct from 'id', 
please be sure that you pass information about id field name in answers ('idField', 'listIdField', 'detailsIdField')`));
    });
    it("should define if attribute required or not", function () {
        let attributes = (0, getEntityAttributes_1.getEntityAttributes)(notNullScalarsDetailsQuery, schema, "id");
        attributes.forEach(attr => (0, chai_1.expect)(attr.required).to.be.true);
        attributes = (0, getEntityAttributes_1.getEntityAttributes)(QUERY_NODE_VISIT_LIST, schema, "id");
        attributes.forEach((attr) => attr.name === "pet"
            ? (0, chai_1.expect)(attr.required).to.be.true
            : (0, chai_1.expect)(attr.required).to.be.false);
    });
    it("should put enum options in entity attributes", function () {
        const attributes = (0, getEntityAttributes_1.getEntityAttributes)(QUERY_WITH_ENUM, schema, "id");
        const defenseStatusAttr = attributes.find(attr => attr.name === "defenseStatus");
        (0, chai_1.expect)(defenseStatusAttr.isEnumType).to.be.true;
        (0, chai_1.expect)(defenseStatusAttr.enumOptions.map(en => en.name).sort())
            .to.eql(["NEEDS_PROTECTION", "NO_DANGER", "RED_BOOK"]);
    });
});
const QUERY_WITHOUT_ID = (0, graphql_tag_1.default)(`
  query Get_Pet($id: BigInteger) {
    pet(id: $id) {
      identificationNumber
      birthDate
    }
  }
`);
const QUERY_WITH_ENUM = (0, graphql_tag_1.default)(`
  query Get_Pet_Type($id: ID!) {
    petType(id: $id) {
      id
      defenseStatus
      name
    }
  }
`);
const QUERY_NODE_VISIT_LIST = (0, graphql_tag_1.default)(`
    query Get_Visit_List {
      visitList {
        description
        id
        pet {
          id
          identificationNumber
          birthDate
        }
        visitEnd
        visitStart
      }
    }`);
const QUERY_NODE_PET_LIST = (0, graphql_tag_1.default)(`
    query Get_Pet_List {
      petList {
        id
        identificationNumber
        birthDate
        photo
        passport
        passportFilename
        type {
          id
          name
        }
        owner {
          id
          firstName
          lastName
        }
        description {
          id
          description
        }
        tags {
          id
          name
        }
        diseases {
          id
          name
          description
        }  
      }
    }
`);
const notNullScalarsDetailsQuery = (0, graphql_tag_1.default)(`
query Get_NN_Scalars($id: ID) {
    notNullScalarsTestEntity(id: $id) {
      id
      bigDecimalNotNull
      bigIntNotNull
      dateTestNotNull
      localDateNotNull
      localDateTimeNotNull
      localTimeNotNull
      offsetDateTimeNotNull
      offsetTimeNotNull
      stringNotNull
      urlNotNull
    }
  }`);
const EXPECTED_ATTRS_PET_LIST = [
    {
        "displayName": "Identification Number",
        "enumOptions": undefined,
        "isMultiRelationField": false,
        "isSingleRelationField": false,
        "kind": "gql",
        "name": "identificationNumber",
        "nestedAttributes": undefined,
        "required": true,
        "type": "String",
        "fileProperties": undefined,
        "isFileType": false,
        "isDateType": false,
        "isEnumType": false,
        "isNumberType": false,
    },
    {
        "displayName": "Birth Date",
        "enumOptions": undefined,
        "isMultiRelationField": false,
        "isSingleRelationField": false,
        "kind": "gql",
        "name": "birthDate",
        "nestedAttributes": undefined,
        "required": false,
        "type": "Date",
        "fileProperties": undefined,
        "isFileType": false,
        "isDateType": true,
        "isEnumType": false,
        "isNumberType": false,
    },
    {
        "displayName": "Photo",
        "enumOptions": undefined,
        "fileProperties": {
            "fileIdSource": "photo"
        },
        "isDateType": false,
        "isEnumType": false,
        "isFileType": true,
        "isMultiRelationField": false,
        "isNumberType": false,
        "isSingleRelationField": false,
        "kind": "gql",
        "name": "photo",
        "nestedAttributes": undefined,
        "required": false,
        "type": "String",
    },
    {
        "displayName": "Passport",
        "enumOptions": undefined,
        "fileProperties": {
            "fileIdSource": "passport"
        },
        "isDateType": false,
        "isEnumType": false,
        "isFileType": true,
        "isMultiRelationField": false,
        "isNumberType": false,
        "isSingleRelationField": false,
        "kind": "gql",
        "name": "passport",
        "nestedAttributes": undefined,
        "required": false,
        "type": "String",
    },
    {
        "displayName": "Passport Filename",
        "enumOptions": undefined,
        "fileProperties": undefined,
        "isDateType": false,
        "isEnumType": false,
        "isFileType": false,
        "isMultiRelationField": false,
        "isNumberType": false,
        "isSingleRelationField": false,
        "kind": "gql",
        "name": "passportFilename",
        "nestedAttributes": undefined,
        "required": false,
        "type": "String",
    },
    {
        "displayName": "Type",
        "enumOptions": undefined,
        "isMultiRelationField": false,
        "isSingleRelationField": true,
        "kind": "gql",
        "name": "type",
        "nestedAttributes": [
            "defenseStatus",
            "id",
            "name",
        ],
        "required": false,
        "type": "PetTypeDTO",
        "fileProperties": undefined,
        "isFileType": false,
        "isDateType": false,
        "isEnumType": false,
        "isNumberType": false,
    },
    {
        "displayName": "Owner",
        "enumOptions": undefined,
        "isMultiRelationField": false,
        "isSingleRelationField": true,
        "kind": "gql",
        "name": "owner",
        "nestedAttributes": [
            "address",
            "city",
            "email",
            "firstName",
            "id",
            "lastName",
            "telephone",
        ],
        "required": false,
        "type": "OwnerDTO",
        "fileProperties": undefined,
        "isFileType": false,
        "isDateType": false,
        "isEnumType": false,
        "isNumberType": false,
    },
    {
        "displayName": "Description",
        "enumOptions": undefined,
        "isMultiRelationField": false,
        "isSingleRelationField": true,
        "kind": "gql",
        "name": "description",
        "nestedAttributes": [
            "description",
            "id",
            "photo",
        ],
        "required": false,
        "type": "PetDescriptionDTO",
        "fileProperties": undefined,
        "isFileType": false,
        "isDateType": false,
        "isEnumType": false,
        "isNumberType": false,
    },
    {
        "displayName": "Tags",
        "enumOptions": undefined,
        "isMultiRelationField": true,
        "isSingleRelationField": false,
        "kind": "gql",
        "name": "tags",
        "nestedAttributes": [
            "id",
            "name",
        ],
        "required": false,
        "type": "TagDTO",
        "fileProperties": undefined,
        "isFileType": false,
        "isDateType": false,
        "isEnumType": false,
        "isNumberType": false,
    },
    {
        "displayName": "Diseases",
        "enumOptions": undefined,
        "isMultiRelationField": true,
        "isSingleRelationField": false,
        "kind": "gql",
        "name": "diseases",
        "nestedAttributes": [
            "description",
            "name",
            "id",
            "treatmentProtocol",
            "treatmentProtocolFilename"
        ],
        "required": false,
        "type": "PetDiseaseDTO",
        "fileProperties": undefined,
        "isFileType": false,
        "isDateType": false,
        "isEnumType": false,
        "isNumberType": false,
    }
];
//# sourceMappingURL=getEntityAttributes.test.js.map