"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.insertSpecifierImport = void 0;
const tslib_1 = require("tslib");
const j = tslib_1.__importStar(require("jscodeshift"));
/**
 * Add SpecifierImport to app file. If SpecifierImport with the same specifierImportName already exist - do nothing.
 * Example of added line `import "react";`;
 *
 * @param appAST ast version of app file, which will be mutated
 * @param specifierImportName name of thing that will be imported from selected module
 * @param moduleName name of the module that will be added as import
 */
function insertSpecifierImport(appAST, specifierImportName, moduleName) {
    if (moduleName.includes("\\")) {
        // Incorrect symbol `\` in module name import path.
        // Symbol '/' must be used as folder separator for all OS (Linux, Windows).
        // Do NOT use `path.join` for import path composition.
        throw new Error(`Incorrect module name '${moduleName}'. Nodejs imports must not contain '\\' symbol.`);
    }
    // no import declaration found - add import to ast
    if (findImportDeclaration(appAST, moduleName).length == 0) {
        const importSpecifiers = appAST
            .find(j.ImportSpecifier)
            .filter(path => path.node.imported.name === specifierImportName);
        // no module with name `moduleName`, but specifier exist
        if (importSpecifiers.length > 0) {
            throw new Error(`Import specifier '${specifierImportName}' already exist in different module`);
        }
        const importDeclaration = j.importDeclaration([], j.stringLiteral(moduleName));
        appAST
            .find(j.Program)
            .forEach(path => path.node.body.unshift(importDeclaration));
    }
    const moduleNameImportCollection = findImportDeclaration(appAST, moduleName);
    // get specifier from declaration
    const importSpecifiers = moduleNameImportCollection
        .find(j.ImportSpecifier)
        .filter(path => path.node.imported.name === specifierImportName);
    // specifier found - no changes required
    if (importSpecifiers.length > 0) {
        console.log(`insertSpecifierImport: skip adding import - import of '${specifierImportName}' from '${moduleName}' already exist`);
        return;
    }
    // add import specifier
    const screenImport = j.importSpecifier(j.identifier(specifierImportName));
    moduleNameImportCollection
        .forEach(path => path.node.specifiers?.push(screenImport));
    console.log(`insertSpecifierImport: insert import of '${specifierImportName}' from '${moduleName}' module to app file`);
}
exports.insertSpecifierImport = insertSpecifierImport;
function findImportDeclaration(appAST, moduleName) {
    return appAST
        .find(j.ImportDeclaration)
        .filter(path => path.value.source.value === moduleName);
}
//# sourceMappingURL=insertSpecifierImport.js.map