"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.read = exports.throwError = exports.normalizeImportRelativePath = exports.isBlank = exports.fqnToName = exports.convertToUnixPath = exports.removeDtoSuffixFromEntityName = exports.splitByCapitalLetter = exports.unCapitalizeFirst = exports.capitalizeFirst = exports.elementNameToClass = void 0;
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const fs = tslib_1.__importStar(require("fs"));
/**
 * Remove illegal symbols and normalize class name by standard js notation.
 *
 * @param {string} elementName 'my-app-custom' | 'myAppCustom' | 'my app custom'
 * @returns {string} class name MyAppCustom
 */
const elementNameToClass = (elementName) => {
    if (elementName.match(/$\s*^/))
        throw new Error('Could not generate class name from empty string');
    return elementName
        .trim().replace(/\s{2,}/g, ' ') // extra spaces
        .replace(/[^\w\d\s_\-$]/g, '') // remove symbols not allowed in class name
        .split(/[-\s]/) // split by separate words
        .map(exports.capitalizeFirst)
        .join('');
};
exports.elementNameToClass = elementNameToClass;
const capitalizeFirst = (part) => part[0].toUpperCase() + part.slice(1);
exports.capitalizeFirst = capitalizeFirst;
const unCapitalizeFirst = (part) => part[0].toLowerCase() + part.slice(1);
exports.unCapitalizeFirst = unCapitalizeFirst;
const splitByCapitalLetter = (word) => word.replace(/([^A-Z])([A-Z])/g, '$1 $2');
exports.splitByCapitalLetter = splitByCapitalLetter;
/**
 * Removes 'Dto' and 'DTO' strings from the end of `word`.
 * @param word
 */
const removeDtoSuffixFromEntityName = (word) => word.length > 3
    ? word.replace(/DTO$|Dto$/, "")
    : word;
exports.removeDtoSuffixFromEntityName = removeDtoSuffixFromEntityName;
function convertToUnixPath(input) {
    const isExtendedLengthPath = /^\\\\\?\\/.test(input);
    const hasNonAscii = /[^\u0000-\u0080]+/.test(input); // eslint-disable-line no-control-regex
    if (isExtendedLengthPath || hasNonAscii) {
        return input;
    }
    return input.replace(/\\/g, '/');
}
exports.convertToUnixPath = convertToUnixPath;
/**
 * Convert java class fully qualified name to compilable TS class name
 *
 * @param fqn java class fqn
 */
function fqnToName(fqn) {
    return fqn.replace(/\./g, '_');
}
exports.fqnToName = fqnToName;
function isBlank(str) {
    return (str == null || /^\s*$/.test(str));
}
exports.isBlank = isBlank;
/**
 *
 * Converts any type of paths (with all types of slashes) to posix path, which is used
 * in typescript imports. This type of path is not depends on current OS (Windows\Linux)
 * and always use forward slashes.
 *
 * Returned path contains only forward slashes and no backward slashes,
 * and has a trailing forward slash.
 *
 * Note, `addLeadingSlash = false` introduced for backward compatibility with antd version of generators.
 * This parameter should be removed after finish antd version support.
 * In react-admin generators this method should be used with `addLeadingSlash = true` only.
 *
 * @param relPath
 * @param addLeadingSlash if `true` - prepend `./` for paths, which are not start from `.`
 */
// TODO check usage and refactor - verify that we really need path conversion in all places, where function is used
function normalizeImportRelativePath(relPath, addLeadingSlash = false) {
    if (relPath == null || isBlank(relPath)) {
        return '';
    }
    const relPathPosix = path_1.default.posix.join(relPath.replace(/\\/g, '/'), '/');
    if (relPathPosix.startsWith('/')) {
        throw new Error(`Error due relative path transform - relative path '${relPathPosix}' must not start from '/'`);
    }
    if (addLeadingSlash && !relPathPosix.startsWith(".")) {
        return `./${relPathPosix}`;
    }
    return relPathPosix;
}
exports.normalizeImportRelativePath = normalizeImportRelativePath;
/**
 * The preferable method of throwing an error in Yeoman is using Environment.error() method.
 * It will prevent execution of the subsequent code, however, TypeScript compiler doesn't know about that.
 * Therefore, using Environment.error() in a function that returns a value will require e.g. throwing an error manually
 * or returning an empty value.
 * In such cases this helper function can be used. It returns `never`, which means that compiler won't consider
 * the code after this function reachable.
 *
 * @param generator
 * @param error
 */
function throwError(generator, error) {
    generator.env.error(Error(error)); // This will throw an error and the subsequent code will never be reached
    throw error; // This code will never be reached and exists only to allow to return `never`
}
exports.throwError = throwError;
function read(filePath) {
    return fs.readFileSync(filePath, 'utf-8');
}
exports.read = read;
//# sourceMappingURL=utils.js.map