import { GraphQLError } from "graphql";
import { AuthProvider } from "react-admin";
import { LOGIN_URI, LOGOUT_URI } from "../config";

export const authProvider: AuthProvider = {
  login: () => Promise.reject("authProvider:login not used in this type of auth"),
  logout: async () => {
    const url = composeUrl(LOGOUT_URI);
    console.log(`authProvider:logout redirect to ${url}`);
    window.location.href = url;
  },
  checkError: async (error) => {
    // TODO code below assumes that GraphQL server returns
    // {"errors":[{"extensions":{"classification":"UNAUTHORIZED"}}], ...}
    // for not authenticated user
    // and
    // {"errors":[{"extensions":{"classification":"FORBIDDEN"}}], ...}
    // if user has not enough permissions for query.
    // If the server handles errors differently, or has a different response structure,
    // code below should be modified.

    const { graphQLErrors, networkError } = error;
    // redirect to login page if graphql returns UNAUTHORIZED error
    if (
      Array.isArray(graphQLErrors) &&
      graphQLErrors.some((err: GraphQLError) => err.extensions?.classification === "UNAUTHORIZED")
    ) {
      const url = composeUrl(LOGIN_URI);
      window.location.href = url;
      console.log(`authProvider:checkError redirect to '${url}'`);
    }

    // redirect to login page for network error with 401 status
    if (networkError?.statusCode === 401) {
      const url = composeUrl(LOGIN_URI);
      window.location.href = url;
      console.log(`authProvider:checkError redirect to '${url}'`);
    }

    console.log("authProvider:checkError - not handle this type of error");
    return Promise.resolve();
  },
  checkAuth: () => {
    console.log(
      "authProvider:checkAuth do nothing - this check is on server responsibility in this type of auth"
    );
    return Promise.resolve();
  },
  getPermissions: () => {
    console.log("authProvider:getPermissions do nothing - not supported now for this type of auth");
    return Promise.resolve();
  },
  getUserIdentity: async () =>
    Promise.reject("authProvider:getUserIdentity not used in this type of auth"),
};

function composeUrl(url: string) {
  return `${url}?redirect_uri=${window.location.href}`;
}
