"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deriveComponentModelList = void 0;
const parseTextFieldDescriptor_1 = require("./descriptors/parseTextFieldDescriptor");
const parseTextAreaDescriptor_1 = require("./descriptors/parseTextAreaDescriptor");
const parseNumberDescriptor_1 = require("./descriptors/parseNumberDescriptor");
const parseCheckListDescriptor_1 = require("./descriptors/parseCheckListDescriptor");
const parseCheckBoxDescriptor_1 = require("./descriptors/parseCheckBoxDescriptor");
const parseDateTimeDescriptor_1 = require("./descriptors/parseDateTimeDescriptor");
const parseSelectDescriptor_1 = require("./descriptors/parseSelectDescriptor");
const parseRadioButtonGroupDescriptor_1 = require("./descriptors/parseRadioButtonGroupDescriptor");
const parseAutocompleteArrayDescriptor_1 = require("./descriptors/parseAutocompleteArrayDescriptor");
const parseDateDescriptor_1 = require("./descriptors/parseDateDescriptor");
const parseTimeDescriptor_1 = require("./descriptors/parseTimeDescriptor");
function deriveComponentModelList(components) {
    const simpleComps = components
        .flatMap((cmp) => {
        if (cmp.type === "group") {
            const { components } = cmp;
            if (components != null && Array.isArray(components) && components.length > 0) {
                components[0].groupStart = true;
                components[0].groupLabel = cmp.label;
                components[0].showOutline = cmp.showOutline;
                components[components.length - 1].groupEnd = true;
                return components;
            }
            return [];
        }
        return cmp;
    })
        .filter((cmp) => 
    // "button" should be handled separately
    // "image", "spacer", "text" - not supported in UI
    cmp != null && !["button", "image", "spacer", "text"].includes(cmp.type));
    return simpleComps
        .map((comp) => {
        const { type, groupStart, groupEnd, groupLabel, showOutline, } = comp;
        return {
            ...deriveComponentProps(comp),
            type,
            groupStart,
            groupLabel,
            groupEnd,
            showOutline,
        };
    });
}
exports.deriveComponentModelList = deriveComponentModelList;
function deriveComponentProps(comp) {
    switch (comp.type) {
        case "textfield":
            return {
                props: (0, parseTextFieldDescriptor_1.parseTextFieldDescriptor)(comp),
            };
        case "textarea":
            return {
                props: (0, parseTextAreaDescriptor_1.parseTextAreaDescriptor)(comp),
            };
        case "number":
            return {
                props: (0, parseNumberDescriptor_1.parseNumberDescriptor)(comp),
            };
        case "datetime":
            switch (comp.subtype) {
                case "datetime":
                    return {
                        subtype: comp.subtype,
                        props: (0, parseDateTimeDescriptor_1.parseDateTimeDescriptor)(comp),
                    };
                case "date":
                    return {
                        subtype: comp.subtype,
                        props: (0, parseDateDescriptor_1.parseDateDescriptor)(comp),
                    };
                case "time":
                    return {
                        subtype: comp.subtype,
                        props: (0, parseTimeDescriptor_1.parseTimeDescriptor)(comp),
                    };
                default:
                    throw new Error(`camunda component with type "${comp.type}" ` +
                        `and subtype "${comp.subtype}" not supported`);
            }
        case "checklist":
            return {
                props: (0, parseCheckListDescriptor_1.parseCheckListDescriptor)(comp),
            };
        case "checkbox":
            return {
                props: (0, parseCheckBoxDescriptor_1.parseCheckBoxDescriptor)(comp),
            };
        case "group":
            throw new Error(`camunda component with type "group" should not exist in array, ` +
                `but should be flattened on previous step`);
        case "button":
            throw new Error(`camunda component with type "button" should be sorted out ` +
                ` on previous step, and handled separately`);
        case "taglist":
            return {
                props: (0, parseAutocompleteArrayDescriptor_1.parseAutocompleteArrayDescriptor)(comp),
            };
        case "select":
            return {
                props: (0, parseSelectDescriptor_1.parseSelectDescriptor)(comp),
            };
        case "radio":
            return {
                props: (0, parseRadioButtonGroupDescriptor_1.parseRadioButtonGroupDescriptor)(comp),
            };
        case "spacer":
        case "text":
        case "image":
            throw new Error(`camunda component with type "${comp.type}" not supported and ` +
                `should be sorted out on previous step`);
        default:
            throw new Error(`camunda component with type "${comp.type}" not supported`);
    }
}
//# sourceMappingURL=deriveComponentModelList.js.map