"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const parseCheckBoxDescriptor_1 = require("./parseCheckBoxDescriptor");
const chai_1 = require("chai");
const parseCheckListDescriptor_1 = require("./parseCheckListDescriptor");
const parseTextFieldDescriptor_1 = require("./parseTextFieldDescriptor");
const parseTextAreaDescriptor_1 = require("./parseTextAreaDescriptor");
const parseNumberDescriptor_1 = require("./parseNumberDescriptor");
const parseDateTimeDescriptor_1 = require("./parseDateTimeDescriptor");
const parseSelectDescriptor_1 = require("./parseSelectDescriptor");
const parseAutocompleteArrayDescriptor_1 = require("./parseAutocompleteArrayDescriptor");
const parseRadioButtonGroupDescriptor_1 = require("./parseRadioButtonGroupDescriptor");
const parseDateDescriptor_1 = require("./parseDateDescriptor");
const parseTimeDescriptor_1 = require("./parseTimeDescriptor");
describe("camunda form descriptors parsing", () => {
    it("parseTextAreaDescriptor", () => {
        const descriptor = {
            "label": "Text",
            "type": "textarea",
            "layout": {
                "row": "Row_04gdzdy",
                "columns": null
            },
            "id": "Field_0mzzvyb",
            "key": "text",
            "disabled": false
        };
        const props = (0, parseTextAreaDescriptor_1.parseTextAreaDescriptor)(descriptor);
        (0, chai_1.expect)(props).to.deep.eq({
            source: 'text',
            label: 'Text',
            helperText: undefined,
            defaultValue: undefined,
            readOnly: false,
            disabled: false,
            validation: {}
        });
    });
    it("parseTextFieldDescriptor", () => {
        const descriptor = {
            "label": "To",
            "type": "textfield",
            "layout": {
                "row": "Row_02k62cb",
                "columns": null
            },
            "id": "Field_14nil3w",
            "key": "to",
            "validate": {
                "validationType": "email",
                "required": true
            }
        };
        const props = (0, parseTextFieldDescriptor_1.parseTextFieldDescriptor)(descriptor);
        (0, chai_1.expect)(props).to.deep.eq({
            source: 'to',
            label: 'To',
            helperText: undefined,
            defaultValue: undefined,
            readOnly: false,
            disabled: false,
            validation: {
                email: true,
                maxLength: undefined,
                minLength: undefined,
                regex: undefined,
                required: true,
            }
        });
    });
    it("parseNumberDescriptor", () => {
        const descriptor = {
            "label": "Retries count",
            "type": "number",
            "layout": {
                "row": "Row_18buy6d",
                "columns": null
            },
            "id": "Field_1uw4mqx",
            "key": "retires_number",
            "description": "Count of retries to send an email",
            "defaultValue": 1,
            "increment": "1",
            "validate": {
                "min": 1,
                "max": 5
            }
        };
        const props = (0, parseNumberDescriptor_1.parseNumberDescriptor)(descriptor);
        (0, chai_1.expect)(props).to.deep.eq({
            source: 'retires_number',
            label: 'Retries count',
            helperText: 'Count of retries to send an email',
            defaultValue: 1,
            readOnly: false,
            disabled: false,
            decimalDigits: 0,
            step: '1',
            validation: {
                maxValue: 5,
                minValue: 1,
                required: undefined,
            }
        });
    });
    it("parseDateTimeDescriptor", () => {
        const descriptor = {
            "dateLabel": "Notification date and time",
            "label": "Date time",
            "type": "datetime",
            "layout": {
                "row": "Row_0v721i9",
                "columns": null
            },
            "id": "Field_1tmvjse",
            "key": "notification_date_time",
            "disallowPassedDates": true,
            "timeInterval": 30,
            "use24h": true
        };
        const props = (0, parseDateTimeDescriptor_1.parseDateTimeDescriptor)(descriptor);
        (0, chai_1.expect)(props).to.deep.eq({
            source: 'notification_date_time',
            label: 'Notification date and time ',
            helperText: undefined,
            readOnly: false,
            disabled: false,
            disablePast: true,
            minutesStep: 30,
            ampm: false,
            ampmInClock: false,
            validation: {
                required: false,
            },
        });
    });
    it("parseDateDescriptor", () => {
        const descriptor = {
            "subtype": "date",
            "dateLabel": "Date",
            "label": "Date time",
            "type": "datetime",
            "layout": {
                "row": "Row_1n4pqgy",
                "columns": null
            },
            "id": "Field_11wfqmu",
            "key": "datetime_yi72if"
        };
        const props = (0, parseDateDescriptor_1.parseDateDescriptor)(descriptor);
        (0, chai_1.expect)(props).to.deep.eq({
            source: 'datetime_yi72if',
            disablePast: false,
            readOnly: false,
            disabled: false,
            helperText: undefined,
            label: 'Date ',
            validation: {
                required: false,
            },
        });
    });
    it("parseTimeDescriptor", () => {
        const descriptor = {
            "subtype": "time",
            "dateLabel": "Date",
            "label": "Date time",
            "type": "datetime",
            "layout": {
                "row": "Row_1n4pqgy",
                "columns": null
            },
            "id": "Field_0bacmig",
            "key": "datetime_hwpkav"
        };
        const props = (0, parseTimeDescriptor_1.parseTimeDescriptor)(descriptor);
        (0, chai_1.expect)(props).to.deep.eq({
            source: 'datetime_hwpkav',
            disablePast: false,
            minutesStep: undefined,
            ampm: true,
            ampmInClock: true,
            readOnly: false,
            disabled: false,
            helperText: undefined,
            label: 'Date ',
            validation: {
                required: false,
            },
        });
    });
    it("parseCheckBoxDescriptor", () => {
        const descriptor = {
            "label": "Is Claim accepted?",
            "type": "checkbox",
            "id": "Field_1lg64wu",
            "key": "isClaimAccepted",
            "description": "Mark as checked if the Claim must be accepted",
            "layout": {
                "row": "Row_1n2k5bm"
            },
            "validate": {
                "required": true
            }
        };
        const props = (0, parseCheckBoxDescriptor_1.parseCheckBoxDescriptor)(descriptor);
        (0, chai_1.expect)(props).to.deep.eq({
            source: 'isClaimAccepted',
            readOnly: false,
            disabled: false,
            defaultChecked: false,
            helperText: 'Mark as checked if the Claim must be accepted',
            label: 'Is Claim accepted?',
            validation: {
                required: true,
            },
        });
    });
    it("parseCheckListDescriptor", () => {
        const descriptor = {
            "values": [
                {
                    "label": "12 hours",
                    "value": "12_hours"
                },
                {
                    "label": "1 day",
                    "value": "1_day"
                },
                {
                    "label": "3 days",
                    "value": "3_days"
                }
            ],
            "label": "Remind after",
            "type": "checklist",
            "layout": {
                "row": "Row_04gdzdt",
                "columns": null
            },
            "id": "Field_1918zkp",
            "key": "remind_after"
        };
        const props = (0, parseCheckListDescriptor_1.parseCheckListDescriptor)(descriptor);
        (0, chai_1.expect)(props).to.deep.eq({
            source: 'remind_after',
            choices: [
                { label: '12 hours', value: '12_hours' },
                { label: '1 day', value: '1_day' },
                { label: '3 days', value: '3_days' }
            ],
            optionText: 'label',
            optionValue: 'value',
            disabled: undefined,
            row: false,
            readOnly: undefined,
            helperText: undefined,
            label: 'Remind after',
            translateChoice: false,
            validation: {
                required: false,
            },
        });
    });
    it("parseSelectDescriptor", () => {
        const descriptor = {
            "values": [
                {
                    "label": "John Smith",
                    "value": "John Smith"
                },
                {
                    "label": "Vanessa May",
                    "value": "Vanessa May"
                },
                {
                    "label": "Leo Moreau",
                    "value": "Leo Moreau"
                }
            ],
            "label": "Assigned veterinarian",
            "type": "select",
            "layout": {
                "row": "Row_0u3mery",
                "columns": null
            },
            "id": "Field_0odrfdd",
            "key": "assignedVet"
        };
        const props = (0, parseSelectDescriptor_1.parseSelectDescriptor)(descriptor);
        (0, chai_1.expect)(props).to.deep.eq({
            source: 'assignedVet',
            choices: [
                { label: 'John Smith', value: 'John Smith' },
                { label: 'Vanessa May', value: 'Vanessa May' },
                { label: 'Leo Moreau', value: 'Leo Moreau' }
            ],
            optionText: 'label',
            optionValue: 'value',
            defaultValue: undefined,
            disabled: false,
            helperText: undefined,
            label: 'Assigned veterinarian',
            readOnly: false,
            validation: {
                required: false,
            },
        });
    });
    it("parseAutocompleteArrayDescriptor", () => {
        const descriptor = {
            "values": [
                {
                    "label": "Value",
                    "value": "value"
                }
            ],
            "label": "Tag list",
            "type": "taglist",
            "layout": {
                "row": "Row_0wnezii",
                "columns": null
            },
            "id": "Field_0hr2dvj",
            "key": "taglist_bxezlm"
        };
        const props = (0, parseAutocompleteArrayDescriptor_1.parseAutocompleteArrayDescriptor)(descriptor);
        (0, chai_1.expect)(props).to.deep.eq({
            source: 'taglist_bxezlm',
            readOnly: false,
            choices: [{ label: 'Value', value: 'value' }],
            optionText: 'label',
            optionValue: 'value',
            disabled: false,
            helperText: undefined,
            label: 'Tag list',
            validation: {
                required: false,
            },
        });
    });
    it("parseRadioButtonGroupDescriptor", () => {
        const descriptor = {
            "values": [
                {
                    "label": "Value",
                    "value": "value"
                }
            ],
            "label": "Radio",
            "type": "radio",
            "layout": {
                "row": "Row_142m9br",
                "columns": null
            },
            "id": "Field_03tolcz",
            "key": "radio_gfavvm"
        };
        const props = (0, parseRadioButtonGroupDescriptor_1.parseRadioButtonGroupDescriptor)(descriptor);
        (0, chai_1.expect)(props).to.deep.eq({
            source: 'radio_gfavvm',
            choices: [{ label: 'Value', value: 'value' }],
            defaultValue: undefined,
            optionText: 'label',
            optionValue: 'value',
            disabled: false,
            helperText: undefined,
            label: 'Radio',
            readOnly: false,
            validation: {
                required: false,
            },
        });
    });
});
//# sourceMappingURL=descriptors.test.js.map