"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseMessagesToNode = exports.mergeI18nMessagesWithLocaleFileContent = void 0;
const tslib_1 = require("tslib");
const getI18nMessages_1 = require("../../../../building-blocks/stages/template-model/pieces/getI18nMessages");
const j = tslib_1.__importStar(require("jscodeshift"));
const parseTs_1 = require("../../../../building-blocks/util/parseTs");
const deepmerge_1 = tslib_1.__importDefault(require("deepmerge"));
const toSource_1 = require("../../../../building-blocks/stages/writing/pieces/jsCodeShift/toSource");
/**
 * Take localeFileContent, parse it to jscodeshift object
 * then find variable with localeMessages by messagesVarName
 * then merge (add if message with such key not exist)  default i18n messages
 * with existing messages to locale file in file system
 *
 * @param localeFileContent - string version of locale file from generated app
 * @param messagesVarName - using to find variable with locale messages inside
 * @param defaultI18nMessages - set of default messages of entity, or other item
 * this messages has low priority in merge, and will be added in result set only if there is no
 * messages with such keys in localeFileContent
 * @returns localeFileContent with added default messages which was missed before
 */
function mergeI18nMessagesWithLocaleFileContent(localeFileContent, messagesVarName, defaultI18nMessages) {
    console.log(`mergeI18nMessagesWithLocaleFileContent: messagesVarName='${messagesVarName}'`);
    const source = (0, parseTs_1.parseTs)(localeFileContent);
    const localeVariable = source.findVariableDeclarators(messagesVarName);
    if (localeVariable.length === 0) {
        throw new Error(`'${messagesVarName}' variable not found in i18n file`);
    }
    const messagesObjectExpression = localeVariable.find(j.ObjectExpression);
    if (messagesObjectExpression.length === 0) {
        throw new Error(`'${messagesVarName}' variable not contains object expression in i18n file`);
    }
    messagesObjectExpression
        .at(0)
        .replaceWith((path) => {
        return parseMessagesToNode((0, deepmerge_1.default)(defaultI18nMessages, parseNodeToMessages(path.node) || {}));
    });
    return (0, toSource_1.toSource)(source);
}
exports.mergeI18nMessagesWithLocaleFileContent = mergeI18nMessagesWithLocaleFileContent;
/**
 * Helper util for mergeI18nMessagesWithLocaleFileContent function
 * Existed messages in file with locale messages should be parsed to node for subsequent merge with new messages.
 * @param messages
 */
function parseMessagesToNode(messages) {
    return j.objectExpression(Object.keys(messages)
        // 'deepmerge' library change order of SpreadElement elements.
        // SpreadElement should be first in js objects after deepmerge
        .sort((key) => (messages[key].__type__ === "SpreadElement" ? -1 : 1))
        .map((key) => {
        if (messages[key].__type__ === "SpreadElement") {
            return j.spreadElement(j.identifier(key));
        }
        const withDotsKey = key.includes(".");
        return j.objectProperty(
        // Object property key could be with dots(invalid symbols for just j.identifier).
        // If so we need to wrap it with brackets in generated code
        withDotsKey ? j.stringLiteral(key) : j.identifier(key), typeof messages[key] === "string" ? j.literal(messages[key]) : parseMessagesToNode(messages[key]));
    }));
}
exports.parseMessagesToNode = parseMessagesToNode;
/**
 * Helper util for mergeI18nMessagesWithLocaleFileContent function
 * Parse jscodeshift ObjectExpression node to javascript object with i18n messages
 * @param node
 */
// TODO add test
function parseNodeToMessages(node) {
    // node is object property
    if (j.ObjectProperty.check(node)) {
        const messageKey = j.Identifier.check(node.key)
            ? node.key.name
            : j.StringLiteral.check(node.key) ? node.key.value : undefined;
        if (messageKey == null) {
            return {};
        }
        let messageValue;
        // node is Literal
        if (j.Literal.check(node.value)) {
            messageValue = typeof node.value.value === "string" ? node.value.value : undefined;
            // node is object, but not literal - recursive parseNodeToMessages
        }
        else {
            messageValue = parseNodeToMessages(node.value) || undefined;
        }
        return messageValue != null ? { [messageKey]: messageValue } : {};
    }
    // node is object expression - recursive parsing for nested nodes
    if (j.ObjectExpression.check(node)) {
        return node.properties.reduce((acc, cur) => ({ ...acc, ...parseNodeToMessages(cur) }), {});
    }
    // node is spread element
    if (j.SpreadElement.check(node)) {
        // check that spread argument is identifier
        const messageKey = j.Identifier.check(node.argument) ? node.argument.name : undefined;
        return messageKey != null ? { [messageKey]: new getI18nMessages_1.SpreadElement() } : {};
    }
    // not supported type of node - return empty object
    return {};
}
//# sourceMappingURL=mergeI18nMessagesWithLocaleFileContent.js.map