"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const client_validation_1 = require("./client-validation");
const chai_1 = require("chai");
const test_commons_1 = require("../../../test/test-commons");
const graphql_tag_1 = tslib_1.__importDefault(require("graphql-tag"));
describe('deriveClientValidationRules()', () => {
    it('correctly derives the rules from GraphQL schema', async () => {
        const schema = await (0, test_commons_1.getMockSchema)();
        const node = (0, graphql_tag_1.default)(MUTATION);
        const { clientValidationRules, validatorImports } = (0, client_validation_1.deriveClientValidationRules)(node, schema);
        (0, chai_1.expect)(clientValidationRules).to.deep.eq(EXPECTED_RULES);
        (0, chai_1.expect)(validatorImports.sort()).to.deep.eq(EXPECTED_IMPORTS.sort());
    });
    it('Update_Owner', async () => {
        const schema = await (0, test_commons_1.getMockSchema)();
        const node = (0, graphql_tag_1.default)(OWNER_MUTATION);
        const { clientValidationRules } = (0, client_validation_1.deriveClientValidationRules)(node, schema);
        (0, chai_1.expect)(clientValidationRules).to.not.be.undefined;
    });
});
describe('getValidatorImport()', () => {
    it('correctly determines validator imports', () => {
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('assertFalse')).to.eq('assertFalse');
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('assertTrue')).to.eq('assertTrue');
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('decimalMax')).to.eq('numeric/compare');
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('decimalMin')).to.eq('numeric/compare');
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('digits')).to.eq('numeric/digits');
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('email')).to.eq(undefined);
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('max')).to.eq('numeric/compare');
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('min')).to.eq('numeric/compare');
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('negative')).to.eq('numeric/negative');
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('negativeOrZero')).to.eq('numeric/negativeOrZero');
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('notBlank')).to.eq(undefined);
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('notEmpty')).to.eq(undefined);
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('notNull')).to.eq(undefined);
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('null')).to.eq('null');
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('pattern')).to.eq(undefined);
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('positive')).to.eq('numeric/positive');
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('positiveOrZero')).to.eq('numeric/positiveOrZero');
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('size')).to.eq(undefined);
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('length')).to.eq(undefined);
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('url')).to.eq(undefined);
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('futureOrPresent', 'date')).to.eq('temporal/futureOrPresentDate');
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('future', 'instant')).to.eq('temporal/futureInstant');
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('pastOrPresent', 'instant')).to.eq('temporal/pastOrPresentInstant');
        (0, chai_1.expect)((0, client_validation_1.getValidatorImport)('past', 'instant')).to.eq('temporal/pastInstant');
    });
});
const MUTATION = `mutation Update_Client_Validation_Test_Entity($input: ClientValidationTestEntityInput!) {
    updateClientValidationTestEntity(input: $input) {
      id
    }
  }`;
const OWNER_MUTATION = 'mutation Update_Owner($input: OwnerInputDTO) {\n    updateOwner(input: $input) {\n      id\n    }\n  }';
const EXPECTED_RULES = {
    businessEmail: [{ rule: 'email', regexp: '.*@haulmont.com$' }],
    email: [{ rule: 'email', regexp: '.*' }],
    eulaAccepted: [
        { rule: 'assertTrue' },
        { rule: 'notNull' }
    ],
    future: [{ rule: 'future', type: 'instant' }],
    futureOrPresent: [{ rule: 'futureOrPresent', type: 'date' }],
    length: [{ rule: 'length', min: 3, max: 5 }],
    negative: [
        { rule: 'negative' },
        { rule: 'notNull' }
    ],
    negativeOrZero: [
        { rule: 'negativeOrZero' },
        { rule: 'notNull' }
    ],
    notBlank: [
        { rule: 'notBlank' },
        { rule: 'notNull' }
    ],
    notEmpty: [
        { rule: 'notEmpty' },
        { rule: 'notNull' }
    ],
    nullField: [{ rule: 'null' }],
    past: [{ rule: 'past', type: 'instant' }],
    pastOrPresent: [{ rule: 'pastOrPresent', type: 'instant' }],
    pattern: [{ rule: 'pattern', regexp: '^A.*' }],
    positive: [{ rule: 'positive' }],
    positiveOrZero: [{ rule: 'positiveOrZero' }],
    price: [
        { rule: 'decimalMax', value: '199.99', inclusive: true },
        { rule: 'decimalMin', value: '150', inclusive: false },
        { rule: 'digits', integer: 3, fraction: 2 }
    ],
    quantity: [
        { rule: 'max', value: 10000 },
        { rule: 'min', value: 500 }
    ],
    requiredWithoutDirective: [{ rule: 'notNull' }],
    size: [{ rule: 'size', min: 3, max: 5, isList: false }],
    trialExpired: [{ rule: 'assertFalse' }],
    urlString: [{ rule: 'url' }],
    urlWithoutDirective: [{ rule: 'url' }]
};
const EXPECTED_IMPORTS = [
    'assertFalse',
    'assertTrue',
    'temporal/futureInstant',
    'temporal/futureOrPresentDate',
    'temporal/pastInstant',
    'temporal/pastOrPresentInstant',
    'numeric/negative',
    'numeric/negativeOrZero',
    'numeric/positive',
    'numeric/positiveOrZero',
    'numeric/compare',
    'numeric/digits',
    'null',
];
//# sourceMappingURL=client-validation.test.js.map